@@ -24,7 +24,7 @@ extern char * child_tty(void);
 extern char * foreground_prog(void);  // to be free()d
 extern void user_command(wstring commands, int n);
 extern wstring child_conv_path(wstring, bool adjust_dir);
-extern void child_fork(int argc, char * argv[], int moni, bool config_size);
+extern void child_fork(int argc, char * argv[], int moni, bool config_size, bool in_cwd);
 extern void child_set_fork_dir(char *);
 extern void setenvi(char * env, int val);
 extern void child_launch(int n, int argc, char * argv[], int moni);
